// This file was generated by the Gtk# code generator.
// Any changes made will be lost if regenerated.

namespace Poppler {

	using System;
	using System.Runtime.InteropServices;

#region Autogenerated code
	[Flags]
	[GLib.GType (typeof (Poppler.AnnotFlagGType))]
	public enum AnnotFlag {

		Unknown,
		Invisible = 1 << 0,
		Hidden = 1 << 1,
		Print = 1 << 2,
		NoZoom = 1 << 3,
		NoRotate = 1 << 4,
		NoView = 1 << 5,
		ReadOnly = 1 << 6,
		Locked = 1 << 7,
		ToggleNoView = 1 << 8,
		LockedContents = 1 << 9,
	}

	internal class AnnotFlagGType {
		[DllImport ("poppler-glib")]
		static extern IntPtr poppler_annot_flag_get_type ();

		public static GLib.GType GType {
			get {
				return new GLib.GType (poppler_annot_flag_get_type ());
			}
		}
	}
#endregion
}
