// This file was generated by the Gtk# code generator.
// Any changes made will be lost if regenerated.

namespace Poppler {

	using System;

	public delegate bool AttachmentSaveFunc(string buf, ulong count);

}
