#region PDFsharp - A .NET library for processing PDF
//
// Authors:
//   Stefan Lange (mailto:Stefan.Lange@pdfsharp.com)
//
// Copyright (c) 2005-2009 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
// http://sourceforge.net/projects/pdfsharp
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
#endregion

namespace PdfSharp.Drawing
{
  ///<summary>
  /// Specifies all pre-defined colors. Used to identify the pre-defined colors and to 
  /// localize their names.
  /// </summary>
  public enum XKnownColor
  {
    /// <summary>A pre-defined color.</summary>
    AliceBlue = 0,

    /// <summary>A pre-defined color.</summary>
    AntiqueWhite = 1,

    /// <summary>A pre-defined color.</summary>
    Aqua = 2,

    /// <summary>A pre-defined color.</summary>
    Aquamarine = 3,

    /// <summary>A pre-defined color.</summary>
    Azure = 4,

    /// <summary>A pre-defined color.</summary>
    Beige = 5,

    /// <summary>A pre-defined color.</summary>
    Bisque = 6,

    /// <summary>A pre-defined color.</summary>
    Black = 7,

    /// <summary>A pre-defined color.</summary>
    BlanchedAlmond = 8,

    /// <summary>A pre-defined color.</summary>
    Blue = 9,

    /// <summary>A pre-defined color.</summary>
    BlueViolet = 10,

    /// <summary>A pre-defined color.</summary>
    Brown = 11,

    /// <summary>A pre-defined color.</summary>
    BurlyWood = 12,

    /// <summary>A pre-defined color.</summary>
    CadetBlue = 13,

    /// <summary>A pre-defined color.</summary>
    Chartreuse = 14,

    /// <summary>A pre-defined color.</summary>
    Chocolate = 15,

    /// <summary>A pre-defined color.</summary>
    Coral = 16,

    /// <summary>A pre-defined color.</summary>
    CornflowerBlue = 17,

    /// <summary>A pre-defined color.</summary>
    Cornsilk = 18,

    /// <summary>A pre-defined color.</summary>
    Crimson = 19,

    /// <summary>A pre-defined color.</summary>
    Cyan = 20,

    /// <summary>A pre-defined color.</summary>
    DarkBlue = 21,

    /// <summary>A pre-defined color.</summary>
    DarkCyan = 22,

    /// <summary>A pre-defined color.</summary>
    DarkGoldenrod = 23,

    /// <summary>A pre-defined color.</summary>
    DarkGray = 24,

    /// <summary>A pre-defined color.</summary>
    DarkGreen = 25,

    /// <summary>A pre-defined color.</summary>
    DarkKhaki = 26,

    /// <summary>A pre-defined color.</summary>
    DarkMagenta = 27,

    /// <summary>A pre-defined color.</summary>
    DarkOliveGreen = 28,

    /// <summary>A pre-defined color.</summary>
    DarkOrange = 29,

    /// <summary>A pre-defined color.</summary>
    DarkOrchid = 30,

    /// <summary>A pre-defined color.</summary>
    DarkRed = 31,

    /// <summary>A pre-defined color.</summary>
    DarkSalmon = 32,

    /// <summary>A pre-defined color.</summary>
    DarkSeaGreen = 33,

    /// <summary>A pre-defined color.</summary>
    DarkSlateBlue = 34,

    /// <summary>A pre-defined color.</summary>
    DarkSlateGray = 35,

    /// <summary>A pre-defined color.</summary>
    DarkTurquoise = 36,

    /// <summary>A pre-defined color.</summary>
    DarkViolet = 37,

    /// <summary>A pre-defined color.</summary>
    DeepPink = 38,

    /// <summary>A pre-defined color.</summary>
    DeepSkyBlue = 39,

    /// <summary>A pre-defined color.</summary>
    DimGray = 40,

    /// <summary>A pre-defined color.</summary>
    DodgerBlue = 41,

    /// <summary>A pre-defined color.</summary>
    Firebrick = 42,

    /// <summary>A pre-defined color.</summary>
    FloralWhite = 43,

    /// <summary>A pre-defined color.</summary>
    ForestGreen = 44,

    /// <summary>A pre-defined color.</summary>
    Fuchsia = 45,

    /// <summary>A pre-defined color.</summary>
    Gainsboro = 46,

    /// <summary>A pre-defined color.</summary>
    GhostWhite = 47,

    /// <summary>A pre-defined color.</summary>
    Gold = 48,

    /// <summary>A pre-defined color.</summary>
    Goldenrod = 49,

    /// <summary>A pre-defined color.</summary>
    Gray = 50,

    /// <summary>A pre-defined color.</summary>
    Green = 51,

    /// <summary>A pre-defined color.</summary>
    GreenYellow = 52,

    /// <summary>A pre-defined color.</summary>
    Honeydew = 53,

    /// <summary>A pre-defined color.</summary>
    HotPink = 54,

    /// <summary>A pre-defined color.</summary>
    IndianRed = 55,

    /// <summary>A pre-defined color.</summary>
    Indigo = 56,

    /// <summary>A pre-defined color.</summary>
    Ivory = 57,

    /// <summary>A pre-defined color.</summary>
    Khaki = 58,

    /// <summary>A pre-defined color.</summary>
    Lavender = 59,

    /// <summary>A pre-defined color.</summary>
    LavenderBlush = 60,

    /// <summary>A pre-defined color.</summary>
    LawnGreen = 61,

    /// <summary>A pre-defined color.</summary>
    LemonChiffon = 62,

    /// <summary>A pre-defined color.</summary>
    LightBlue = 63,

    /// <summary>A pre-defined color.</summary>
    LightCoral = 64,

    /// <summary>A pre-defined color.</summary>
    LightCyan = 65,

    /// <summary>A pre-defined color.</summary>
    LightGoldenrodYellow = 66,

    /// <summary>A pre-defined color.</summary>
    LightGray = 67,

    /// <summary>A pre-defined color.</summary>
    LightGreen = 68,

    /// <summary>A pre-defined color.</summary>
    LightPink = 69,

    /// <summary>A pre-defined color.</summary>
    LightSalmon = 70,

    /// <summary>A pre-defined color.</summary>
    LightSeaGreen = 71,

    /// <summary>A pre-defined color.</summary>
    LightSkyBlue = 72,

    /// <summary>A pre-defined color.</summary>
    LightSlateGray = 73,

    /// <summary>A pre-defined color.</summary>
    LightSteelBlue = 74,

    /// <summary>A pre-defined color.</summary>
    LightYellow = 75,

    /// <summary>A pre-defined color.</summary>
    Lime = 76,

    /// <summary>A pre-defined color.</summary>
    LimeGreen = 77,

    /// <summary>A pre-defined color.</summary>
    Linen = 78,

    /// <summary>A pre-defined color.</summary>
    Magenta = 79,

    /// <summary>A pre-defined color.</summary>
    Maroon = 80,

    /// <summary>A pre-defined color.</summary>
    MediumAquamarine = 81,

    /// <summary>A pre-defined color.</summary>
    MediumBlue = 82,

    /// <summary>A pre-defined color.</summary>
    MediumOrchid = 83,

    /// <summary>A pre-defined color.</summary>
    MediumPurple = 84,

    /// <summary>A pre-defined color.</summary>
    MediumSeaGreen = 85,

    /// <summary>A pre-defined color.</summary>
    MediumSlateBlue = 86,

    /// <summary>A pre-defined color.</summary>
    MediumSpringGreen = 87,

    /// <summary>A pre-defined color.</summary>
    MediumTurquoise = 88,

    /// <summary>A pre-defined color.</summary>
    MediumVioletRed = 89,

    /// <summary>A pre-defined color.</summary>
    MidnightBlue = 90,

    /// <summary>A pre-defined color.</summary>
    MintCream = 91,

    /// <summary>A pre-defined color.</summary>
    MistyRose = 92,

    /// <summary>A pre-defined color.</summary>
    Moccasin = 93,

    /// <summary>A pre-defined color.</summary>
    NavajoWhite = 94,

    /// <summary>A pre-defined color.</summary>
    Navy = 95,

    /// <summary>A pre-defined color.</summary>
    OldLace = 96,

    /// <summary>A pre-defined color.</summary>
    Olive = 97,

    /// <summary>A pre-defined color.</summary>
    OliveDrab = 98,

    /// <summary>A pre-defined color.</summary>
    Orange = 99,

    /// <summary>A pre-defined color.</summary>
    OrangeRed = 100,

    /// <summary>A pre-defined color.</summary>
    Orchid = 101,

    /// <summary>A pre-defined color.</summary>
    PaleGoldenrod = 102,

    /// <summary>A pre-defined color.</summary>
    PaleGreen = 103,

    /// <summary>A pre-defined color.</summary>
    PaleTurquoise = 104,

    /// <summary>A pre-defined color.</summary>
    PaleVioletRed = 105,

    /// <summary>A pre-defined color.</summary>
    PapayaWhip = 106,

    /// <summary>A pre-defined color.</summary>
    PeachPuff = 107,

    /// <summary>A pre-defined color.</summary>
    Peru = 108,

    /// <summary>A pre-defined color.</summary>
    Pink = 109,

    /// <summary>A pre-defined color.</summary>
    Plum = 110,

    /// <summary>A pre-defined color.</summary>
    PowderBlue = 111,

    /// <summary>A pre-defined color.</summary>
    Purple = 112,

    /// <summary>A pre-defined color.</summary>
    Red = 113,

    /// <summary>A pre-defined color.</summary>
    RosyBrown = 114,

    /// <summary>A pre-defined color.</summary>
    RoyalBlue = 115,

    /// <summary>A pre-defined color.</summary>
    SaddleBrown = 116,

    /// <summary>A pre-defined color.</summary>
    Salmon = 117,

    /// <summary>A pre-defined color.</summary>
    SandyBrown = 118,

    /// <summary>A pre-defined color.</summary>
    SeaGreen = 119,

    /// <summary>A pre-defined color.</summary>
    SeaShell = 120,

    /// <summary>A pre-defined color.</summary>
    Sienna = 121,

    /// <summary>A pre-defined color.</summary>
    Silver = 122,

    /// <summary>A pre-defined color.</summary>
    SkyBlue = 123,

    /// <summary>A pre-defined color.</summary>
    SlateBlue = 124,

    /// <summary>A pre-defined color.</summary>
    SlateGray = 125,

    /// <summary>A pre-defined color.</summary>
    Snow = 126,

    /// <summary>A pre-defined color.</summary>
    SpringGreen = 127,

    /// <summary>A pre-defined color.</summary>
    SteelBlue = 128,

    /// <summary>A pre-defined color.</summary>
    Tan = 129,

    /// <summary>A pre-defined color.</summary>
    Teal = 130,

    /// <summary>A pre-defined color.</summary>
    Thistle = 131,

    /// <summary>A pre-defined color.</summary>
    Tomato = 132,

    /// <summary>A pre-defined color.</summary>
    Transparent = 133,

    /// <summary>A pre-defined color.</summary>
    Turquoise = 134,

    /// <summary>A pre-defined color.</summary>
    Violet = 135,

    /// <summary>A pre-defined color.</summary>
    Wheat = 136,

    /// <summary>A pre-defined color.</summary>
    White = 137,

    /// <summary>A pre-defined color.</summary>
    WhiteSmoke = 138,

    /// <summary>A pre-defined color.</summary>
    Yellow = 139,

    /// <summary>A pre-defined color.</summary>
    YellowGreen = 140,
  }
}
