// This file was generated by the Gtk# code generator.
// Any changes made will be lost if regenerated.

namespace Poppler {

	using System;
	using System.Runtime.InteropServices;

#region Autogenerated code
	[Flags]
	[GLib.GType (typeof (Poppler.ViewerPreferencesGType))]
	public enum ViewerPreferences {

		Unset,
		HideToolbar = 1 << 0,
		HideMenubar = 1 << 1,
		HideWindowui = 1 << 2,
		FitWindow = 1 << 3,
		CenterWindow = 1 << 4,
		DisplayDocTitle = 1 << 5,
		DirectionRtl = 1 << 6,
	}

	internal class ViewerPreferencesGType {
		[DllImport ("poppler-glib")]
		static extern IntPtr poppler_viewer_preferences_get_type ();

		public static GLib.GType GType {
			get {
				return new GLib.GType (poppler_viewer_preferences_get_type ());
			}
		}
	}
#endregion
}
