#region PDFsharp - A .NET library for processing PDF
//
// Authors:
//   Stefan Lange (mailto:Stefan.Lange@pdfsharp.com)
//
// Copyright (c) 2005-2009 empira Software GmbH, Cologne (Germany)
//
// http://www.pdfsharp.com
// http://sourceforge.net/projects/pdfsharp
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
#endregion

using System;
using System.Globalization;
using System.Diagnostics;
using System.Collections;
using PdfSharp.Internal;
using PdfSharp.Drawing;
using PdfSharp.Pdf;
using PdfSharp.Pdf.IO;
using PdfSharp.Pdf.Internal;

namespace PdfSharp.Pdf.Annotations
{
  /// <summary>
  /// Represents a text annotation.
  /// </summary>
  internal sealed class PdfWidgetAnnotation : PdfAnnotation
  {
    public PdfWidgetAnnotation()
    {
      Initialize();
    }

    public PdfWidgetAnnotation(PdfDocument document)
      : base(document)
    {
      Initialize();
    }

    void Initialize()
    {
      Elements.SetName(Keys.Subtype, "/Widget");
    }

    /// <summary>
    /// Predefined keys of this dictionary.
    /// </summary>
    internal new class Keys : PdfAnnotation.Keys
    {
      /// <summary>
      /// (Optional) The annotation’s highlighting mode, the visual effect to be used when
      /// the mouse button is pressed or held down inside its active area:
      ///   N (None) No highlighting.
      ///   I (Invert) Invert the contents of the annotation rectangle.
      ///   O (Outline) Invert the annotation’s border.
      ///   P (Push) Display the annotation’s down appearance, if any. If no down appearance is defined,
      ///     offset the contents of the annotation rectangle to appear as if it were being pushed below
      ///     the surface of the page.
      ///   T (Toggle) Same as P (which is preferred).
      /// A highlighting mode other than P overrides any down appearance defined for the annotation. 
      /// Default value: I.
      /// </summary>
      [KeyInfo(KeyType.Name | KeyType.Optional)]
      public const string H = "/H";

      /// <summary>
      /// (Optional) An appearance characteristics dictionary to be used in constructing a dynamic 
      /// appearance stream specifying the annotation’s visual presentation on the page.
      /// The name MK for this entry is of historical significance only and has no direct meaning.
      /// </summary>
      [KeyInfo(KeyType.Dictionary | KeyType.Optional)]
      public const string MK = "/MK";

      public static DictionaryMeta Meta
      {
        get
        {
          if (Keys.meta == null)
            Keys.meta = CreateMeta(typeof(Keys));
          return Keys.meta;
        }
      }
      static DictionaryMeta meta;
    }

    /// <summary>
    /// Gets the KeysMeta of this dictionary type.
    /// </summary>
    internal override DictionaryMeta Meta
    {
      get { return Keys.Meta; }
    }
  }
}
