// This file was generated by the Gtk# code generator.
// Any changes made will be lost if regenerated.

namespace Poppler {

	using System;
	using System.Collections;
	using System.Runtime.InteropServices;

#region Autogenerated code
	public class Action : GLib.Object {

		[Obsolete]
		protected Action(GLib.GType gtype) : base(gtype) {}
		public Action(IntPtr raw) : base(raw) {}

		protected Action() : base(IntPtr.Zero)
		{
			CreateNativeObject (new string [0], new GLib.Value [0]);
		}

		[DllImport("poppler-glib")]
		static extern IntPtr poppler_action_copy(IntPtr raw);

		public Poppler.Action Copy() {
			IntPtr raw_ret = poppler_action_copy(Handle);
			Poppler.Action ret = GLib.Object.GetObject(raw_ret, true) as Poppler.Action;
			return ret;
		}

		[DllImport("poppler-glib")]
		static extern void poppler_action_free(IntPtr raw);

		public void Free() {
			poppler_action_free(Handle);
		}

		[DllImport("poppler-glib")]
		static extern IntPtr poppler_action_get_type();

		public static new GLib.GType GType { 
			get {
				IntPtr raw_ret = poppler_action_get_type();
				GLib.GType ret = new GLib.GType(raw_ret);
				return ret;
			}
		}

#endregion
	}
}
