// This file was generated by the Gtk# code generator.
// Any changes made will be lost if regenerated.

namespace Poppler {

	using System;
	using System.Runtime.InteropServices;

#region Autogenerated code
	[GLib.GType (typeof (Poppler.PageModeGType))]
	public enum PageMode {

		Unset,
		None,
		UseOutlines,
		UseThumbs,
		FullScreen,
		UseOc,
		UseAttachments,
	}

	internal class PageModeGType {
		[DllImport ("poppler-glib")]
		static extern IntPtr poppler_page_mode_get_type ();

		public static GLib.GType GType {
			get {
				return new GLib.GType (poppler_page_mode_get_type ());
			}
		}
	}
#endregion
}
