// This file was generated by the Gtk# code generator.
// Any changes made will be lost if regenerated.

namespace Poppler {

	using System;
	using System.Runtime.InteropServices;

#region Autogenerated code
	[GLib.GType (typeof (Poppler.ActionTypeGType))]
	public enum ActionType {

		Unknown,
		None,
		GotoDest,
		GotoRemote,
		Launch,
		Uri,
		Named,
		Movie,
	}

	internal class ActionTypeGType {
		[DllImport ("poppler-glib")]
		static extern IntPtr poppler_action_type_get_type ();

		public static GLib.GType GType {
			get {
				return new GLib.GType (poppler_action_type_get_type ());
			}
		}
	}
#endregion
}
